﻿using Infragistics.Controls.Charts;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Data;

using XamDataChartNoiseDataDemo.Models;

namespace XamDataChartNoiseDataDemo.Converters
{
    public class SelectedSignalsToChartSeriesConverter : IMultiValueConverter
    {
        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            XamDataChart chart = values[0] as XamDataChart;

            if (chart == null)
            {
                return null;
            }

            IList<Models.CrankshaftSignal> cranks = values[1] as IList<Models.CrankshaftSignal>;

            if (cranks != null)
            {

                // remove signals (unselected signals)
                for (int i = chart.Series.Count - 1; i >= 0; i--)
                {
                    if (cranks.FirstOrDefault(item => (chart.Series[i].DataContext == item)) == null)
                    {
                        chart.Series.RemoveAt(i);
                    }
                }


                // add signals (selected signals)
                foreach (Models.CrankshaftSignal crank in cranks)
                {
                    if (chart.Series.Where(s => s.DataContext == crank).FirstOrDefault() == null)
                    {
                        // ScatterLineSeries series = new ScatterLineSeries();
                        // 
                        // series.XMemberPath = nameof(DataPoint.X);
                        // series.YMemberPath = nameof(DataPoint.Y);
                        // series.MarkerType = MarkerType.None;
                        // 
                        // series.SetBinding(ScatterLineSeries.XAxisProperty, new Binding { Source = chart.Axes[0] });
                        // series.SetBinding(ScatterLineSeries.YAxisProperty, new Binding { Source = chart.Axes[1] });
                        // series.SetBinding(ScatterLineSeries.DataContextProperty, new Binding { Source = crank });
                        // series.SetBinding(ScatterLineSeries.ItemsSourceProperty, new Binding { Path = new PropertyPath(nameof(crank.Points)) });
                        // series.SetBinding(ScatterLineSeries.TitleProperty, new Binding { Source = crank, Path = new PropertyPath(nameof(crank.SignalName)) });

                        ImmediateLineSeries.ImmediateLineSeries series = new ImmediateLineSeries.ImmediateLineSeries();
                        
                        series.Thickness = 1;
                        series.Resolution = 0.001;
                        series.UseHairline = true;
                        series.UpscaleOPD = true;
                        series.XMemberPath = nameof(DataPoint.X);
                        series.YMemberPath = nameof(DataPoint.Y);
                        
                        series.SetBinding(ImmediateLineSeries.ImmediateLineSeries.XAxisProperty, new Binding { Source = chart.Axes[0] });
                        series.SetBinding(ImmediateLineSeries.ImmediateLineSeries.YAxisProperty, new Binding { Source = chart.Axes[1] });
                        series.SetBinding(ImmediateLineSeries.ImmediateLineSeries.DataContextProperty, new Binding { Source = crank });
                        series.SetBinding(ImmediateLineSeries.ImmediateLineSeries.ItemsSourceProperty, new Binding { Path = new PropertyPath(nameof(crank.Points)) });
                        series.SetBinding(ImmediateLineSeries.ImmediateLineSeries.TitleProperty, new Binding { Source = crank, Path = new PropertyPath(nameof(crank.SignalName)) });

                        chart.Series.Add(series);
                    }

                }
            }

            return null;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

}
