﻿using Infragistics;
using Infragistics.Controls.Charts;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;
using System.Windows.Shapes;

namespace XamDataChartNoiseDataDemo.ImmediateLineSeries
{
    public class ImmediateLineSeriesBaseView : SeriesView
    {
        protected ImmediateLineSeries OrderedScatterModel { get; set; }

        public bool IsThumbnail { get; set; }

        public ImmediateLineSeriesBaseView MainView { get; set; }

        public ImmediateLineSeriesBaseView(ImmediateLineSeries model)
            : base(model)
        {
            OrderedScatterModel = model;
        }

        public virtual void OnRender(DrawingContext drawingContext)
        {
        }

        public virtual void OnResize(double width, double height)
        {

        }

        protected Path Polyline0 = new Path();
        protected Path Polygon01 = new Path();
        protected Path Polyline1 = new Path();

        internal void Rasterize(double[] x, double[] minY, double[] maxY, bool[] hasValue, List<SegmentInfo> segments, Clipper c, double resolution, bool useHairline)
        {
            SegmentInfo segment;
            Point minPoint;
            Point maxPoint;
            List<Point> currentMin;
            List<Point> currentMax;

            PathGeometry p0 = new PathGeometry();
            PathGeometry p1 = new PathGeometry();
            PathGeometry pg = new PathGeometry();
            int minCount;
            int maxCount;

            for (var j = 0; j < segments.Count; j++)
            {
                segment = segments[j];
                currentMin = new List<Point>();
                currentMax = new List<Point>();

                var newEnd = Squish(x, hasValue, segment.Start, segment.End);
                Squish(minY, hasValue, segment.Start, segment.End);
                Squish(maxY, hasValue, segment.Start, segment.End);

                var flattenedMin = new List<int>();
                
                FastFlattener.FastFlatten(flattenedMin, x, minY, segment.Start, newEnd, resolution);
                var flattenedMax = new List<int>();
                FastFlattener.FastFlatten(flattenedMax, x, maxY, segment.Start, newEnd, resolution);

                c.Target = currentMin;
                for (var i = 0; i < flattenedMin.Count; i++)
                {
                    c.Add(new Point(x[flattenedMin[i]], minY[flattenedMin[i]]));
                }
                c.Target = null;

                c.Target = currentMax;
                for (var i = 0; i < flattenedMax.Count; i++)
                {
                    c.Add(new Point(x[flattenedMax[i]], maxY[flattenedMax[i]]));
                }
                c.Target = null;

                segment.MinPoints = currentMin;
                segment.MaxPoints = currentMax;
            }

            //if (!useHairline)
            {
                for (var j = 0; j < segments.Count; j++)
                {
                    segment = segments[j];

                    PathFigure pf = new PathFigure();
                    PolyLineSegment ps = new PolyLineSegment();
                    currentMin = segment.MinPoints;
                    currentMax = segment.MaxPoints;

                    minCount = currentMin.Count;
                    maxCount = currentMax.Count;

                    if (minCount > 0)
                    {
                        pf.StartPoint = currentMin[0];
                    }
                    if (minCount > 1)
                    {
                        for (var i = 1; i < minCount; i++)
                        {
                            ps.Points.Add(currentMin[i]);
                        }
                    }
                    pf.Segments.Add(ps);
                    p0.Figures.Add(pf);

                    if (!useHairline)
                    {
                        pf = new PathFigure();
                        pf.IsClosed = true;
                        ps = new PolyLineSegment();
                        if (maxCount > 0)
                        {
                            pf.StartPoint = currentMax[0];
                        }
                        if (maxCount > 1)
                        {
                            for (var i = 1; i < maxCount; i++)
                            {
                                ps.Points.Add(currentMax[i]);
                            }
                        }
                        if (maxCount == 0 && minCount > 0)
                        {
                            pf.StartPoint = currentMin[minCount - 1];
                        }
                        if (minCount > 1)
                        {
                            for (var i = minCount - 2; i >= 0; i--)
                            {
                                ps.Points.Add(currentMin[i]);
                            }
                        }

                        pf.Segments.Add(ps);
                        pg.Figures.Add(pf);
                    }
                }
            }

            if (!useHairline)
            {
                for (var j = segments.Count - 1; j >= 0; j--)
                {
                    segment = segments[j];

                    PathFigure pf = new PathFigure();
                    PolyLineSegment ps = new PolyLineSegment();
                    currentMax = segment.MaxPoints;
                    maxCount = currentMax.Count;

                    if (maxCount > 0)
                    {
                        pf.StartPoint = currentMax[maxCount - 1];
                    }
                    if (maxCount > 1)
                    {
                        for (var i = maxCount - 2; i >= 0; i--)
                        {
                            ps.Points.Add(currentMax[i]);
                        }
                    }
                    pf.Segments.Add(ps);
                    p1.Figures.Add(pf);
                }
            }

            Polyline0.Data = p0;
            Polyline1.Data = p1;
            Polygon01.Data = pg;
        }

        internal int Squish(double[] vals, bool[] hasValue, int startIndex, int endIndex)
        {
            int squishIndex = startIndex;
            for (var i = startIndex; i <= endIndex; i++)
            {
                if (hasValue[i])
                {
                    if (squishIndex != i)
                    {
                        vals[squishIndex] = vals[i];
                    }
                    squishIndex++;
                }
            }

            return squishIndex - 1;
        }

        public virtual void OnRendered()
        {

        }
    }
}
